using Godot;
using System;

public class Clock : Node
{
    [Signal]
    delegate void OnTick(int ticks);
    [Signal]
    delegate void OnPauseChanged(bool paused);

    [Export(PropertyHint.None, "Number of ticks per second")]
    private float _tickRate;
    private float TickPeriod => 1.0f / _tickRate;

    [Export]
    private bool _autoTick;

    private int _ticks = 0;
    private bool _paused = false;

    public override void _Input(InputEvent @event)
    {
        base._Input(@event);

        if (@event is InputEventKey keyEvent && keyEvent.Pressed)
        {
            switch ((KeyList)keyEvent.Scancode)
            {
                case KeyList.Space:
                    if (_autoTick)
                    {
                        _paused = !_paused;
                        EmitSignal(nameof(OnPauseChanged), _paused);
                        return;
                    }
                    Tick();
                    break;
            }
        }
    }

    private float _secondsSinceLastTick;

    public override void _Process(float delta)
    {
        base._Process(delta);

        if (!_autoTick)
            return;

        if (_paused)
            return;

        _secondsSinceLastTick += delta;
        if (_secondsSinceLastTick < TickPeriod)
            return;

        _secondsSinceLastTick -= TickPeriod;
        Tick();
    }

    private void Tick()
    {
        _ticks++;
        EmitSignal(nameof(OnTick), _ticks);
    }
}
