using Godot;
using System;

public class GridCursor : Sprite
{
    [Export]
    public NodePath Grid { get; set; }
    private WorldGrid _grid;
    private ShaderMaterial _material;

    #region Animation
    [Export]
    private Curve _pulseShape;
    [Export]
    private float _pulseRate = 1.0f;
    private float PulsePeriod => 1.0f / _pulseRate;
    #endregion

    private float _elapsed = 0.0f;
    private bool _pulsing = false;

    private const string T = "t";

    public int X { get; private set; }
    public int Y { get; private set; }

    public override void _Ready()
    {
        _grid = GetNode<WorldGrid>(Grid);
        _material = (ShaderMaterial)Material;
    }

    public override void _Input(InputEvent @event)
    {
        base._Input(@event);

        if (@event is InputEventMouseMotion mouseMoveEvent)
        {
            var pos = mouseMoveEvent.Position;
            _grid.GetGridPos(pos, out var x, out var y);
            X = x;
            Y = y;

            this.Visible = _grid.IsInBounds(X, Y);
            var position = new Vector2(X + .5f, Y + .5f) * _grid.CellSize;
            this.Position = position;
            return;
        }
    }

    public override void _Process(float delta)
    {
        _elapsed += delta * _pulseRate;

        if (!_pulsing)
        {
            _material.SetShaderParam(T, 1f);
            return;
        }

        var a = _pulseShape.Interpolate(_elapsed % 1.0f);
        _material.SetShaderParam(T, a);
    }

    public void _on_Interaction_Mode_OnInteractionModeChanged(Mode oldMode, Mode newMode)
    {
        _pulsing = newMode is BuildMode;
    }
}
