using Godot;
using System;

public abstract class Mode : Node
{
    [Signal]
    delegate void OnModeEntered();
    [Signal]
    delegate void OnModeExited();

    [Export]
    private KeyList _key;

    public bool Active
    {
        get => _active;
        private set
        {
            if (value == _active)
                return;

            _active = value;

            var signal = _active
                ? nameof(OnModeEntered)
                : nameof(OnModeExited);
            EmitSignal(signal);
        }
    }
    private bool _active;

    public override void _Input(InputEvent @event)
    {
        base._Input(@event);

        if (!(@event is InputEventKey keyEvent))
            return;

        if (!keyEvent.Pressed)
            return;

        var key = (KeyList)keyEvent.Scancode;

        if (key == _key)
        {
            Active = true;
        }
    }

    public void Enable()
    {
        Active = true;
        OnEnabled();
    }
    protected abstract void OnEnabled();

    public void Disable()
    {
        Active = false;
        OnDisabled();
    }
    protected abstract void OnDisabled();

    public override string ToString()
    {
        return GetType().ToString();
    }
}
